from flask import Blueprint, redirect, render_template, request, url_for

from CTFd.constants.config import ChallengeVisibilityTypes, Configs
from CTFd.utils.config import is_teams_mode
from CTFd.utils.dates import ctf_ended, ctf_paused, ctf_started
from CTFd.utils.decorators import (
    during_ctf_time_only,
    require_complete_profile,
    require_verified_emails,
)
from CTFd.utils.decorators.visibility import check_challenge_visibility
from CTFd.utils.helpers import get_errors, get_infos
from CTFd.utils.user import authed, get_current_team
from CTFd.models import Challenges

challenges = Blueprint("challenges", __name__)


@challenges.route("/challenges", methods=["GET"])
@require_complete_profile
@during_ctf_time_only
@require_verified_emails
@check_challenge_visibility
def listing(page=None):
    # if is_admin():
    #         chal = Challenges.query.filter(Challenges.id == challenge_id).first_or_404()
    # else:
    #     chal = Challenges.query.filter(
    #         Challenges.id == challenge_id,
    #         and_(Challenges.state != "hidden", Challenges.state != "locked"),
    #     ).first_or_404()

    if (
        Configs.challenge_visibility == ChallengeVisibilityTypes.PUBLIC
        and authed() is False
    ):
        pass
    else:
        if is_teams_mode() and get_current_team() is None:
            return redirect(url_for("teams.private", next=request.full_path))

    infos = get_infos()
    errors = get_errors()

    if Configs.challenge_visibility == ChallengeVisibilityTypes.ADMINS:
        infos.append("La visibilité des défis est configuré uniquement pour les administrateurs")

    if ctf_started() is False:
        errors.append(f"{Configs.ctf_name} n'a pas encore commencé, les défis apparaîtrons sur cette page")

    if ctf_paused() is True:
        infos.append(f"{Configs.ctf_name} est en pause")

    if ctf_ended() is True:
        infos.append(f"{Configs.ctf_name} est terminé")

    return render_template("challenges.html", infos=infos, errors=errors)
