import Alpine from "alpinejs";
import dayjs from "dayjs";

import CTFd from "./index";

import CommentBox from "./components/comments/CommentBox.vue";
import Vue from "vue";
import { Modal, Tab, Tooltip } from "bootstrap";
import highlight from "./theme/highlight";
import favicon from ".";
import { default as helpers } from "./compat/helpers";
import { el } from "lolight";

window.values = [];
window.TEAM_ID = CTFd.team.id;
window.USER_ID = CTFd.user.id;
function addTargetBlank(html) {
  let dom = new DOMParser();
  let view = dom.parseFromString(html, "text/html");
  let links = view.querySelectorAll('a[href*="://"]');
  links.forEach(link => {
    link.setAttribute("target", "_blank");
  });
  return view.documentElement.outerHTML;
}

window.Alpine = Alpine;

Alpine.store("challenge", {
  data: {
    view: "",
  },
});

Alpine.data("Hint", () => ({
  id: null,
  html: null,

  async showHint(event) {
    if (event.target.open) {
      let response = await CTFd.pages.challenge.loadHint(this.id);
      let hint = response.data;
      if (hint.content) {
        this.html = addTargetBlank(hint.html);
      } else {
        let answer = await CTFd.pages.challenge.displayUnlock(this.id);
        if (answer) {
          let unlock = await CTFd.pages.challenge.loadUnlock(this.id);

          if (unlock.success) {
            let response = await CTFd.pages.challenge.loadHint(this.id);
            let hint = response.data;
            this.html = addTargetBlank(hint.html);
          } else {
            event.target.open = false;
            CTFd._functions.challenge.displayUnlockError(unlock);
          }
        } else {
          event.target.open = false;
        }
      }
    }
  },
}));

Alpine.data("Challenge", () => ({
  id: null,
  next_id: null,
  submission: "",
  tab: null,
  solves: [],
  response: null,
  share_url: null,

  async init() {
    highlight();
  },

  getStyles() {
    let styles = {
      "modal-dialog": true,
    };
    try {
      let size = CTFd.config.themeSettings.challenge_window_size;
      switch (size) {
        case "sm":
          styles["modal-sm"] = true;
          break;
        case "lg":
          styles["modal-lg"] = true;
          break;
        case "xl":
          styles["modal-xl"] = true;
          break;
        default:
          break;
      }
    } catch (error) {
      // Ignore errors with challenge window size
      console.log("Erreur de traitement challenge_window_size");
      console.log(error);
    }
    return styles;
  },

  async init() {
    highlight();
  },

  async showChallenge() {
    new Tab(this.$el).show();
  },

  async showSolves() {
    this.solves = await CTFd.pages.challenge.loadSolves(this.id);
    console.log("showSolvers (challenge.js) exexuction");
    console.log(this.solves);
    this.solves.forEach(solve => {
      solve.date = dayjs(solve.date).format("MMMM Do, h:mm:ss A");
      return solve;
    });
    new Tab(this.$el).show();
  },

  async showComments() {
    const commentBox = Vue.extend(CommentBox);
    let vueContainer = document.createElement("div");
    document
      .querySelector("#comment-box")
      .removeChild(document.querySelector("#comment-box").firstChild);
    document.querySelector("#comment-box").appendChild(vueContainer);
    new commentBox({
      propsData: { type: "team", id: window.TEAM_ID, challenge_id: this.id },
    }).$mount(vueContainer);
    new Tab(this.$el).show();
  },

  getNextId() {
    let data = Alpine.store("challenge").data;
    return data.next_id;
  },

  async nextChallenge() {
    let modal = Modal.getOrCreateInstance("[x-ref='challengeWindow']");

    // TODO: Get rid of this private attribute access
    // See https://github.com/twbs/bootstrap/issues/31266
    modal._element.addEventListener(
      "hidden.bs.modal",
      event => {
        // Dispatch load-challenge event to call loadChallenge in the ChallengeBoard
        Alpine.nextTick(() => {
          this.$dispatch("load-challenge", this.getNextId());
        });
      },
      { once: true },
    );
    modal.hide();
  },

  async getShareUrl() {
    let body = {
      type: "solve",
      challenge_id: this.id,
    };
    const response = await CTFd.fetch("/api/v1/shares", {
      method: "POST",
      body: JSON.stringify(body),
    });
    const data = await response.json();
    const url = data["data"]["url"];
    this.share_url = url;
  },

  copyShareUrl() {
    alert();
    // si pas sous http faut work around
    if (!window.isSecureContext) {
      const textArea = document.createElement("textarea");
      textArea.value = text;
      document.body.appendChild(textArea);
      textArea.focus();
      textArea.select();
      try {
        document.execCommand("copy");
      } catch (err) {
        console.error("Impossible de copier dans le presse-papiers", err);
      }
      document.body.removeChild(textArea);
    } else {
      navigator.clipboard.writeText(this.share_url);
      let t = Tooltip.getOrCreateInstance(this.$el);
      t.enable();
      t.show();
      setTimeout(() => {
        t.hide();
        t.disable();
      }, 2000);
    }
  },

  async submitChallenge() {
    console.log(this.submission);
    this.response = await CTFd.pages.challenge.submitChallenge(
      this.id,
      this.submission,
    );

    await this.renderSubmissionResponse();
  },

  async renderSubmissionResponse() {
    if (this.response.data.status === "correct") {
      this.submission = "";
    }

    // Dispatch load-challenges event to call loadChallenges in the ChallengeBoard
    this.$dispatch("load-challenges");
  },

  async submitManualChallenge(type) {
    //dans le cas ou le user envoit des fichier

    if (!document.getElementById("file-input").hidden) {
      let form = document.getElementById("form-file-input");
      document.getElementById("form-file-input").value = this;
      var formData = new FormData(form);
      // output as an object
      if (Object.fromEntries(formData)["file"].name != "") {
        try {
          await helpers.files.upload(
            form,
            { id: this.id, type: type },
            async function (response) {
              let thiis = document.getElementById("form-file-input").value;

              thiis.$dispatch("load-challenges");
            },
          );
        } catch (error) {
          this.response = {};
          this.response.data = {};
          this.response.data.status = "incorrect";
          this.response.data.message =
            "Une erreur s'est produite, veuillez contacter l'administrateur pour " +
            error;
          this.$dispatch("load-challenges");
          document.getElementById("challenge-submit").disabled = false;
        }
      } else {
        alert("Vous essayez actuellement d'envoyer rien.");
        document.getElementById("challenge-submit").disabled = false;
      }
    }
    //dans le cas ou le user envoit du text
    else {
      if (document.getElementById("text-input").value != "") {
        this.response = await CTFd.pages.challenge.submitChallenge(
          this.id,
          document.getElementById("text-input").value,
        );
        if (this.response.success) {
          this.response.data.status = "correct";
          this.response.data.message = "Envoyé avec succès!";
        } else {
          this.response.data.status = "incorrect";
          this.response.data.message =
            "Une erreur s'est produite veuillez contacter votre administrateur";
        }

        this.$dispatch("load-challenges");
      } else {
        alert("Vous essayez actuellement d'envoyer rien.");
        document.getElementById("challenge-submit").disabled = false;
      }
    }
    document.getElementById("file-input").textContent =
      "Veuillez sélectionner un fichier";
    document.getElementById("text-input").value = "";
  },

  async submitSportChallenge(challengeId) {
    //dans le cas ou le user envoit des fichier
    let fileInput = document.getElementById("form-file-input-btn");
    let form = document.getElementById("form-file-input");
    document.getElementById("this-holder").value = this;
    // Ensure the form exists
    if (!form || !fileInput || fileInput.hidden) {
      alert("Erreur: Aucun fichier à envoyer ou formulaire manquant.");
      return;
    }

    // Create FormData object
    let formData = new FormData(form);
    form
    let valueId = `value-${challengeId}`;
    let value = document.getElementById(valueId);
    let currentSum = value ? value.textContent : null;

    let challenge = await CTFd.pages.challenges.getChallenge(challengeId);    

    const userValueInput = document.getElementById("unit-input");
    const userValue = userValueInput ? userValueInput.value : null;

    let submissionScore = Math.round(userValue) * challenge.value;
    if (userValue) {
      let hiddenUserValueInput = form.querySelector('input[name="userValue"]');
      if (!hiddenUserValueInput) {
        hiddenUserValueInput = document.createElement("input");
        hiddenUserValueInput.type = "hidden";
        hiddenUserValueInput.name = "userValue";
        form.appendChild(hiddenUserValueInput);
      }
      hiddenUserValueInput.value = submissionScore;
    }
    let sum = Number(currentSum) + Number(submissionScore);
    if(sum <= challenge.max_points && submissionScore > 0){
      if (formData.has("file") && formData.get("file").name !== "") {
        try {
          await helpers.files.upload(
            form,
            { id: this.id, type: "sport" },
            async function (response) {
              console.log(response);
              const newSumId = `new-sum-${challengeId}`;
              let newSumElement = document.getElementById(newSumId);

              if (!newSumElement) {
                newSumElement = document.createElement("div");
                newSumElement.id = newSumId;
                newSumElement.style.display = "none"; // Hide the element
                document.body.appendChild(newSumElement); // Append to the body or a specific container
              }

              // Update the text content of the hidden element
              newSumElement.textContent = sum;
              this.$dispatch("load-challenges");
            },
          );
        } catch (error) {
          this.response = {};
          this.response.data = {};
          this.response.data.status = "incorrect";
          this.response.data.message =
            "Une erreur s'est produite, veuillez contacter l'administrateur pour " +
            error;
          this.$dispatch("load-challenges");
          document.getElementById("challenge-submit").disabled = false;
        }
      } else {
        alert("Vous essayez actuellement d'envoyer rien.");
        document.getElementById("challenge-submit").disabled = false;
      }
    } else if(Math.round(challenge.max_points-currentSum) < challenge.value) {
      alert("Le nombre de points maximal a été atteint pour ce défi");
    } else {
      alert("Vous devez entrer une valeur entre 1 et " + Math.floor(challenge.max_points/challenge.value-currentSum/challenge.value));
    }
    document.getElementById("file-input").textContent =
      "Veuillez sélectionner un fichier";
    document.getElementById("text-input").value = "";
  },
}));

Alpine.data("ChallengeBoard", () => ({
  loaded: false,
  challenges: [],
  challenge: null,

  async init() {
    let args = {};
    args[`team_id`] = window.TEAM_ID;
    let apiArgs = args;
    
    apiArgs[`page`] = 1;
    apiArgs[`per_page`] = 10000;

    this.challenges = await CTFd.pages.challenges.getChallenges();
    //retirer du code 2025-01-04 car fesait du trouble
    //this.submissions = await CTFd.pages.submissions.getSubmissions(args);
    let standings = await CTFd.pages.scoreboard.getScoreboard();
    // let response = await CTFd.fetch(`/api/v1/scoreboard/top/${count}`, {
    //   method: "GET",
    // });
    // const body = await response.json();
    // let ScoreboardDetail = body["data"];
    // let ScoreboardDetail = await CTFd.pages.scoreboard.getScoreboardDetail(
    //   standings.length,
    // );
    this.comments = await helpers.comments.get_comments(apiArgs);
    this.maxScore = 0;
    this.solveScore = 0;
    this.submitScore = 0;
    for (let challenge in this.challenges) {
      let score = this.challenges[challenge].value;
      if (this.challenges[challenge].solved_by_me) {
        this.solveScore += score;
      } else if (this.challenges[challenge].submited) {
        this.submitScore += score;
      }
      this.maxScore += score;

      if (this.challenges[challenge].type == "sport") {
        let sum = 0;
        for (let submission in this.submissions) {
          if (
            this.submissions[submission].challenge_id ==
              this.challenges[challenge].id &&
            (this.challenges[challenge].solved_by_me ||
              this.challenges[challenge].submited)
          ) {
            sum += this.submissions[submission].value;
          }
        }
        this.challenges[challenge].value = sum;
      }
      let valueId = `value-${this.challenges[challenge].id}`;
      let value = document.getElementById(valueId);
      let currentSum = value ? value.textContent : null;

      let maxPointsId = `max-points-${this.challenges[challenge].id}`;
      let maxPointsElem = document.getElementById(maxPointsId);
      let max_points = maxPointsElem ? maxPointsElem.textContent : null;

      if(currentSum != null && max_points != null && currentSum == max_points){
        let button = document.querySelector(`button.challenge-button[value="${this.challenges[challenge].id}"]`);
        button.classList.add("challenge-solved");
      }
    }

    document.getElementById("scoreProgressTitle").textContent =
      this.solveScore + this.submitScore + " points";
    document.getElementById("scoreProgressBar").value =
      100 * ((this.solveScore + this.submitScore) / this.maxScore);
    let text;
    if (this.solveScore != 0) {
      text =
        "" +
        parseInt(100 * (this.submitScore / (this.solveScore + this.submitScore))) +
        "% des points en approbations";
    } else {
      text =
        "Un de vos défis doit être validé pour que vos soumissions soient visibles par les autres équipes.";
    }
    document.getElementById("scoreProgressText").textContent = text;

    this.commentsChallengeDict = {};

    for (let i in this.comments.data) {
      let message = this.comments.data[i].content;
      if (message.search("#") != -1) {
        message = message.split("#")[1];
        let fin = message.search(":");
        let id = 0;
        if (fin != -1) {
          id = parseInt(message.split(":")[0]);
        } else {
          id = parseInt(message.split(":")[0]);
        }

        if (this.commentsChallengeDict[id] != undefined) {
          this.commentsChallengeDict[id] = this.commentsChallengeDict[id] + 1;
        } else {
          this.commentsChallengeDict[id] = 1;
        }
      }
    }

    for (let i in this.commentsChallengeDict) {
      try {
        document.getElementById(i).textContent =
          this.commentsChallengeDict[i] > 99
            ? 99
            : this.commentsChallengeDict[i] <= 9
              ? "  " + this.commentsChallengeDict[i]
              : this.commentsChallengeDict[i];
        document.getElementById(i).className +=
          " fas fa-comments float-start challenge-comments";
      } catch {}
    }

    this.loaded = true;

    if (window.location.hash) {
      let chalHash = decodeURIComponent(window.location.hash.substring(1));
      let idx = chalHash.lastIndexOf("-");
      if (idx >= 0) {
        let pieces = [chalHash.slice(0, idx), chalHash.slice(idx + 1)];
        let id = pieces[1];
        await this.loadChallenge(id);
      }
    }
  },
  getCategories() {
    const categories = [];

    this.challenges.forEach(challenge => {
      const { category } = challenge;

      if (!categories.includes(category)) {
        categories.push(category);
      }
    });

    try {
      const f = CTFd.config.themeSettings.challenge_category_order;
      if (f) {
        const getSort = new Function(`return (${f})`);
        categories.sort(getSort());
      }
    } catch (error) {
      // Ignore errors with theme category sorting
      console.log("Erreur lors de l'exécution de la fonction challenge_category_order");
      console.log(error);
    }

    return categories;
  },
  getChallenges(category) {
    let challenges = this.challenges;

    if (category !== null) {
      challenges = this.challenges.filter(challenge => challenge.category === category);
    }

    try {
      const f = CTFd.config.themeSettings.challenge_order;
      if (f) {
        const getSort = new Function(`return (${f})`);
        challenges.sort(getSort());
      }
    } catch (error) {
      // Ignore errors with theme challenge sorting
      console.log("Erreur lors de l'exécution de la fonction challenge_order");
      console.log(error);
    }
    console.log(challenges);
    return challenges;
  },

  async loadChallenges() {
    this.challenges = await CTFd.pages.challenges.getChallenges();
    for (let challenge in this.challenges) {
      let score = this.challenges[challenge].value;
      if (this.challenges[challenge].solved_by_me) {
        this.solveScore += score;
      } else if (this.challenges[challenge].submited) {
        this.submitScore += score;
      }
      this.maxScore += score;

      const newSumId = `new-sum-${this.challenges[challenge].id}`;
      const newSum = document.getElementById(newSumId);
      const newSumUpdate = newSum ? newSum.textContent : null;
      if(newSumUpdate != null){
        this.challenges[challenge].value = sum;
      }
      else if (this.challenges[challenge].type == "sport") {
        let sum = 0;
        for (let submission in this.submissions) {
          if (
            this.submissions[submission].challenge_id ==
              this.challenges[challenge].id &&
            (this.challenges[challenge].solved_by_me ||
              this.challenges[challenge].submited)
          ) {
            sum += this.submissions[submission].value;
          }
        }
        this.challenges[challenge].value = sum;
      }

    }
  },

  async loadChallenge(challengeId) {
    await CTFd.pages.challenge.displayChallenge(challengeId, challenge => {
      challenge.data.view = addTargetBlank(challenge.data.view);

      Alpine.store("challenge").data = challenge.data;

      // nextTick is required here because we're working in a callback
      Alpine.nextTick(() => {
        let modal = Modal.getOrCreateInstance("[x-ref='challengeWindow']");
        // TODO: Get rid of this private attribute access
        // See https://github.com/twbs/bootstrap/issues/31266
        modal._element.addEventListener(
          "hidden.bs.modal",
          event => {
            // Remove location hash
            history.replaceState(null, null, " ");
          },
          { once: true },
        );
        modal.show();
        history.replaceState(null, null, `#${challenge.data.name}-${challengeId}`);
      });
    });
  },
}));

Alpine.start();
(() => {
  var now = Date.now();

  var f = (element, seconds) => {
    if (seconds <= 0) {
      element.innerHTML = "&nbsp;";

      element.parentElement.parentElement.disabled = true;
      element.parentElement.parentElement.className += " disabled";
      element.className = element.className.split(" time")[0];
      console.log(element);
      element.parentElement.parentElement.parentElement.getElementsByClassName(
        "flash-effect",
      )[0].className = "";
      element.parentElement.parentElement.parentElement.getElementsByClassName(
        "flash-image",
      )[0].src = "/themes/core-beta/static/img/flashStable.png";
      element.className = "";
      element.parentElement.parentElement.getElementsByClassName(
        "challenge-submit",
      )[0].disabled = true;
      element.className = "";
      return;
    }
    var days = (seconds / 86400) | 0;
    var hours = ((seconds % 86400) / 3600) | 0;
    var minutes = ((seconds % 3600) / 60) | 0;
    var seconds = seconds % 60 | 0;

    element.textContent = days + ":" + hours + ":" + minutes + ":" + seconds;
  };

  setInterval(() => {
    var elapsed = (Date.now() / 1000) | 0;
    var elements = document.getElementsByClassName("time");

    for (var i = 0; i < elements.length; i++) {
      var element = elements[i];
      var seconds = element.id - elapsed;
      console.log(element.id);

      f(element, seconds);
    }
  }, 1000);
})();
this.hit = function () {
  let fileInput = document.getElementById("file-input");
  let textInput = document.getElementById("text-input");
  let mediaLabel = document.getElementById("soumettre-media-label");
  let textLabel = document.getElementById("soumettre-texte-label");

  if (textInput.hidden) {
    fileInput.hidden = true;
    textInput.hidden = false;
    mediaLabel.hidden = true;
    textLabel.hidden = false;
  } else {
    fileInput.hidden = false;
    textInput.hidden = true;
    mediaLabel.hidden = false;
    textLabel.hidden = true;
  }
};

this.changeLabel = function (event) {
  let output = event.target.files.length + " fichier(s) à envoyer";
  let totalSize = 0;
  let formats_video = [
    "hevc",
    "mp4",
    "avi",
    "mkv",
    "mov",
    "MOV",
    "wmv",
    "flv",
    "webm",
    "mpeg",
    "3gp",
    "ogv",
  ];
  let formats_image = [
    "jpeg",
    "png",
    "bmp",
    "tiff",
    "webp",
    "raw",
    "heic",
    "ico",
    "jpg",
  ];

  for (let i = 0; i < event.target.files.length; i++) {
    let extension = event.target.files[i].name.split(".")[1];

    totalSize += event.target.files[i].size;
    if (
      !formats_video.includes(extension.toLowerCase()) &&
      !formats_image.includes(extension.toLowerCase())
    ) {
      alert("Nous ne supportons pas l'extension ." + extension);
      event.target.value = "";
      output = "Sélectionnez un fichier";
    }
  }

  if (totalSize > 100000000 && totalSize < 200000000) {
    alert(
      "Les fichiers envoyés sont au dessus de 100MB. Ils seront compressés. [" +
        totalSize / 1000000 +
        "/100MB]",
    );
  } else if (totalSize > 200000000) {
    alert(
      "Les fichiers envoyés dépassent 200MB. Veuillez utiliser un outil externe et partager un lien. [" +
        totalSize / 1000000 +
        "/200MB]",
    );
    event.target.value = "";
    output = "Sélectionnez un fichier";
  }
  if (event.target.files.length > 20) {
    alert("Vous ne pouvez pas envoyer plus de 20 médias à la fois.");
    event.target.value = "";
    output = "Sélectionnez un fichier";
  }
  document.getElementById("file-input").textContent = output;
};

this.hit = function () {
  let fileInput = document.getElementById("file-input");
  let textInput = document.getElementById("text-input");
  let mediaLabel = document.getElementById("soumettre-media-label");
  let textLabel = document.getElementById("soumettre-texte-label");

  if (textInput.hidden) {
    fileInput.hidden = true;
    textInput.hidden = false;
    mediaLabel.hidden = true;
    textLabel.hidden = false;
  } else {
    fileInput.hidden = false;
    textInput.hidden = true;
    mediaLabel.hidden = false;
    textLabel.hidden = true;
  }
};

const submission_api = {
  getSubmissions: function (id) {
    return CTFd.api.get_submissions_list({ challengeId: id }).then(response => {
      console.log("Submissions for id" + id);
      const data = response.data;
      // $(".challenge-solves").text(parseInt(data.length) + " Solves");
      // const box = $("#challenge-solves-names");
      // box.empty();
      // for (let i = 0; i < data.length; i++) {
      //   const account_id = data[i].account_id;
      //   const name = data[i].name;
      //   const date = dayjs(data[i].date).fromNow();
      //   const account_url = data[i].account_url;
      //   box.append(
      //     '<tr><td><a href="{0}">{2}</td><td>{3}</td></tr>'.format(
      //       account_url,
      //       account_id,
      //       htmlEntities(name),
      //       date
      //     )
      //   );
      // }
    });
  },
};
